(*$U-,R-,V-,S-*)
PROGRAM DumbTrm;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*   Program:   DumbTrm                                                     *)
(*                                                                          *)
(*   Author:    Philip R. Burns                                             *)
(*                                                                          *)
(*   Date:      November 11, 1987.                                          *)
(*              Revised May 23, 1988.                                       *)
(*                                                                          *)
(*   Purpose:   Dumb terminal emulator to demonstrate PibAsync routines.    *)
(*                                                                          *)
(*   Usage:     Compile to .EXE file using Turbo Pascal and execute the     *)
(*              .EXE file by typing  DUMBTRM  at the DOS prompt.            *)
(*                                                                          *)
(*              You will be prompted for the required communications        *)
(*              parameters.  If the serial port can be opened successfully, *)
(*              then a dumb terminal mode is entered.                       *)
(*                                                                          *)
(*              To exit dumb terminal mode, type ^C (ascii 03).             *)
(*                                                                          *)
(*   Needs:     See USES below.                                             *)
(*                                                                          *)
(*   Remarks:   This program doesn't check for input mistakes when the      *)
(*              communications parameters are being entered.                *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

USES
   Crt, Dos, GlobType, PibTimer, PibAsync;

VAR
   Baud_Rate   : WORD              (* Baud rate for connection, e.g., 1200  *);
   Com_Port    : INTEGER           (* Which port, e.g., 1 for COM1:         *);
   Parity      : CHAR              (* Parity, e.g., E for even parity       *);
   Data_Bits   : INTEGER           (* How many bits per character, e.g., 8  *);
   Stop_Bits   : INTEGER           (* How many stop bits -- nearly always 1 *);

VAR
   InBufSize   : INTEGER           (* Size of input buffer                  *);
   OutBufSize  : INTEGER           (* Size of output buffer                 *);
   Do_XonXoff  : CHAR              (* 'Y' to do XON/XOFF flow control       *);
   Do_HardWired: CHAR              (* 'Y' to do XON/XOFF flow control       *);
   Do_CTS      : CHAR              (* 'Y' to do CTS/RTS flow control        *);
   Do_DSR      : CHAR              (* 'Y' to do DSR/DTR flow control        *);

VAR
   Com_Addr_Str: STRING[10]        (* Comm port address in hex string       *);
   Com_Addr    : INTEGER           (* Comm port address                     *);
   Com_Irq     : INTEGER           (* Comm port IRQ address                 *);
   Com_Int_No  : INTEGER           (* Comm port interrupt vector number     *);

(*----------------------------------------------------------------------*)
(*         Hex_To_Dec --- Convert hex string to decimal number          *)
(*----------------------------------------------------------------------*)

FUNCTION Hex_To_Dec( S : AnyStr; Default : INTEGER ) : INTEGER;

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*     Function:   Hex_To_Dec                                           *)
(*                                                                      *)
(*     Purpose:    Convert hex string to decimal number                 *)
(*                                                                      *)
(*     Calling Sequence:                                                *)
(*                                                                      *)
(*        Integ := Hex_To_Dec( S: AnyStr; Default: INTEGER ) : INTEGER; *)
(*                                                                      *)
(*           S       --- the hex string                                 *)
(*           Default --- value to return if S not hex string            *)
(*           Integ   --- corresponding decimal integer (0 if bad)       *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

VAR
   I  : INTEGER;
   Sum: INTEGER;

BEGIN (* Hex_To_Dec *)

   Sum        := 0;
   Hex_To_Dec := Default;

   FOR I := 1 TO LENGTH( S ) DO
      CASE S[I] OF
         '0'..'9':  Sum := Sum * 16 + ( ORD(S[I]) - ORD('0') );
         'A'..'F':  Sum := Sum * 16 + ( ORD(S[I]) - ORD('A') + 10 );
         ELSE       EXIT;
      END;

   Hex_To_Dec := Sum;

END   (* Hex_To_Dec *);

(*--------------------------------------------------------------------------*)

PROCEDURE Get_Comm_Params;

VAR
   YesNo : CHAR;

BEGIN (* Get_Comm_Params *)
                                   (* No direct screen writes *)
   DirectVideo := FALSE;

   WRITELN;

   WRITE('Enter com port (1,2,3,4):                             ');
   READLN( Com_Port );

   IF ( Com_Port > 2 ) THEN
      BEGIN

         WRITE('Enter com port base address (hex):                    ');
         READLN( Com_Addr_Str );

         Com_Addr := Hex_To_Dec( Com_Addr_Str , -1 );

         WRITE('Enter com port IRQ level (1 to 15):                   ');
         READLN( Com_Irq );

         WRITE('Enter com port interrupt vector number (hex):         ');
         READLN( Com_Addr_Str );

         Com_Int_No := Hex_To_Dec( Com_Addr_Str , -1 );

      END;

   WRITE('Enter baud rate (1200,2400,9600,19200,38400,57600):   ');
   READLN( Baud_Rate );

   WRITE('Enter parity (E=even, N=one, O=odd, M=mark, S=space): ');
   READLN( Parity );

   Parity := UpCase( Parity );

   WRITE('Enter bits per character (7 or 8):                    ');
   READLN( Data_Bits );

   WRITE('Enter stop bits (usually 1):                          ');
   READLN( Stop_Bits );

   WRITE('Enter size in bytes of async receive buffer:          ');
   READLN( InBufSize );

   WRITE('Enter size in bytes of async output buffer:           ');
   READLN( OutBufSize );

   WRITE('Use XON/XOFF flow control (Y/N)?                      ');
   READLN( Do_XonXoff );

   WRITE('Do CTS/RTS flow control (Y/N)?                        ');
   READLN( Do_CTS );

   WRITE('Do DSR/DTR flow control (Y/N)?                        ');
   READLN( Do_DSR );

   WRITE('Is connection hard-wired (Y/N)?                       ');
   READLN( Do_HardWired );

END   (* Get_Comm_Params *);

(*--------------------------------------------------------------------------*)

FUNCTION Initialize_Communications : BOOLEAN;

BEGIN (* Initialize_Communications *)

                                   (* Set CTS checking *)

   Async_Do_CTS         := ( UpCase( Do_CTS ) = 'Y' );

                                   (* Set DSR checking *)

   Async_Do_DSR         := ( UpCase( Do_DSR ) = 'Y' );

                                   (* Set XON/XOFF to user request *)

   Async_Do_XonXoff     := ( UpCase( Do_XonXoff ) = 'Y' );

                                   (* Set hard-wired as user requests *)

   Async_Hard_Wired_On  := ( UpCase( Do_HardWired ) = 'Y' );

                                   (* Set half-second break duration *)
   Async_Break_Length   := 500;

                                   (* Let XON/XOFF break points default. *)

   Async_Init( InBufSize, OutBufSize, 0, 0, 0);

                                   (* If com port 3 or 4, make sure     *)
                                   (* port address specified in memory. *)
   IF ( Com_Port > 2 ) THEN
      Async_Setup_Port( Com_Port, Com_Addr, Com_Irq, Com_Int_No );

                                   (* Try opening the serial port.   *)

   IF ( NOT Async_Open( Com_Port, Baud_Rate, Parity, Data_Bits, Stop_Bits ) ) THEN
      BEGIN
         WRITELN('Cannot open serial port.');
         Initialize_Communications := FALSE;
      END
   ELSE
      BEGIN
         WRITELN('Serial port opened, DumbTrm ready.');
         Initialize_Communications := TRUE;
      END;

END   (* Initialize_Communications *);

(*--------------------------------------------------------------------------*)

PROCEDURE Emulate_Dumb_Terminal;

VAR
   Kch   : CHAR;
   Ch    : CHAR;
   Flags : BYTE;
   Do_Async_Status : BOOLEAN;
   Mute_Mode       : BOOLEAN;

CONST
   Digits : ARRAY[0..15] OF CHAR = ('0','1','2','3','4','5','6','7','8','9',
                                    'A','B','C','D','E','F');

BEGIN (* Emulate_Dumb_Terminal *)
                                   (* Display <CTRL>key commands    *)

   WRITELN('<CTRL>A toggles line error reporting (now OFF).');
   WRITELN('<CTRL>B sends a break.');
   WRITELN('<CTRL>C quits DumbTrm.');
   WRITELN('<CTRL>E toggles mute mode (now OFF).');
   WRITELN('<CTRL>N toggles noise rejection mode (now ON).');

                                   (* Line error reporting off      *)
   Do_Async_Status := FALSE;
                                   (* Mute mode on to start         *)
   Mute_Mode       := FALSE;
                                   (* Begin loop over serial port   *)
                                   (* and keyboard.                 *)
   REPEAT
                                   (* Pick up and display character *)
                                   (* from port, if any.            *)

      IF Async_Receive( Ch ) THEN
         CASE ORD( Ch ) OF
            0: ;
            7: IF ( NOT Mute_Mode ) THEN
                  WRITE( Ch )
               ELSE
                  WRITE( '<BEL>' );
            ELSE WRITE( Ch );
         END (* CASE *);
                                   (* Read character from keyboard  *)
                                   (* and send out port, unless it  *)
                                   (* is <CTRL>key command.         *)
      IF KeyPressed THEN
         BEGIN
            KCh := ReadKey;
            CASE KCh OF

               ^A: BEGIN
                      Do_Async_Status := NOT Do_Async_Status;
                      CASE Do_Async_Status OF
                         TRUE : BEGIN
                                   WRITELN('Line error reporting mode ON');
                                END;
                         FALSE: BEGIN
                                   WRITELN('Line error reporting mode OFF');
                                END;
                      END (* CASE *);
                   END;

               ^B: Async_Send_Break;

               ^C: ;

               ^E: BEGIN
                      Mute_Mode := NOT Mute_Mode;
                      WRITELN;
                      CASE Mute_Mode OF
                         TRUE : BEGIN
                                   WRITELN('Mute mode ON');
                                END;
                         FALSE: BEGIN
                                   WRITELN('Mute mode OFF');
                                END;
                      END (* CASE *);
                   END;

               ^N: BEGIN
                      Async_Reject_Noise := NOT Async_Reject_Noise;
                      WRITELN;
                      CASE Async_Reject_Noise OF
                         TRUE : BEGIN
                                   WRITELN;
                                   WRITE('New noise character: ');
                                   Async_Noise_Char := ReadKey;
                                   WRITELN( Async_Noise_Char );
                                   Async_Set_Rejection( TRUE ,
                                                        Async_Noise_Char );
                                   WRITELN('Noise rejection ON');
                                END;
                         FALSE: BEGIN
                                   Async_Set_Rejection( FALSE ,
                                                        Async_Noise_Char );
                                   WRITELN('Noise rejection OFF');
                                END;
                      END (* CASE *);
                   END;

               ^Q: BEGIN
                      Async_Send_Now( Ch );
                      Async_XOFF_Received := FALSE;
                   END;

               ELSE
                  Async_Send_Now( Kch );

            END (* CASE *);
         END;

      IF Do_Async_Status THEN
         IF Async_Line_Error( Flags ) THEN
            BEGIN
               WRITELN;
               WRITELN( 'Line error = <', Digits[ Flags SHR 4 ],
                                          Digits[ Flags AND $F ], '>' );
            END;

   UNTIL ( Kch = ^C );

END   (* Emulate_Dumb_Terminal *);

(*--------------------------------------------------------------------------*)

PROCEDURE Finish_Communications;

BEGIN (* Finish_Communications *)
                                   (* Close port and drop DTR *)
   Async_Close( FALSE );
                                   (* Release space allocated for buffers *)
   Async_Release_Buffers;

END   (* Finish_Communications *);

(*--------------------------------------------------------------------------*)

BEGIN (* DumbTrm *)
                                   (* Request serial port parameters     *)
   Get_Comm_Params;
                                   (* Initialize port                    *)

   IF Initialize_Communications THEN
      BEGIN
                                   (* Emulate dumb terminal until ^C hit *)
         Emulate_Dumb_Terminal;
                                   (* Close down port                    *)
         Finish_Communications;

      END;

END   (* DumbTrm *).