UNIT Scaling;
{
  unit holding routines to perform autoscaling functions on data arrays
  of various types pointed to by pointers; also will scale data arrays that
  have an associated array of uncertainties ("Error Bars") to make sure that
  the auto scaled range includes the error bars too.

  the main heart of the scaling is from a recent issue of TUG Lines; I've
  just added a more general purpose shell around it.

}
INTERFACE

USES HiGraf;

TYPE
  DataType = (BYTE_ARRAY,WORD_ARRAY,INTEGER_ARRAY,LONGINT_ARRAY,FLOAT_ARRAY);

PROCEDURE Scale_Axis(RArray, RArray_Err :POINTER; MyType :DataType;
                     N_Points :INTEGER;
                     VAR AxisMin, AxisMax, MajorTic :FLOAT;
                     Axis_Type :LINLOG);

IMPLEMENTATION

PROCEDURE Limits_of_Array(RArray,RArray_Err :POINTER; MyType :DataType;
                          N_Points :INTEGER; Axis_Type :LINLOG;
                          VAR x_min,x_max :FLOAT);
{
  procedure to find maximum and minimum extremes of a real array of
  numbers including the effect of error bars
}
TYPE
  ByteArray       = ARRAY [0..1] OF BYTE;
  WordArray       = ARRAY [0..1] OF WORD;
  IntegerArray    = ARRAY [0..1] OF INTEGER;
  LongArray       = ARRAY [0..1] OF LONGINT;
  FloatArray      = ARRAY [0..1] OF FLOAT;
VAR
  index              : INTEGER;
  temp               : FLOAT;
  ByteArrayPtr       : ^ByteArray;
  ByteErrArrayPtr    : ^ByteArray;
  WordArrayPtr       : ^WordArray;
  WordErrArrayPtr    : ^WordArray;
  IntegerArrayPtr    : ^IntegerArray;
  IntegerErrArrayPtr : ^IntegerArray;
  LongArrayPtr       : ^LongArray;
  LongErrArrayPtr    : ^LongArray;
  FloatArrayPtr      : ^FloatArray;
  FloatErrArrayPtr   : ^FloatArray;
BEGIN
  x_min := 1.0e37;
  x_max := -1.0e37;

  { first, align the pointers }
  CASE MyType OF
    Byte_Array    : ByteArrayPtr    := RArray;
    Word_Array    : WordArrayPtr    := RArray;
    Integer_Array : IntegerArrayPtr := RArray;
    LongInt_Array : LongArrayPtr    := RArray;
    Float_Array   : FloatArrayPtr   := RArray;
  END;
  IF (RArray_Err <> NIL) THEN { error array exists, so do it too... }
    CASE MyType OF
      Byte_Array    : ByteErrArrayPtr    := RArray_Err;
      Word_Array    : WordErrArrayPtr    := RArray_Err;
      Integer_Array : IntegerErrArrayPtr := RArray_Err;
      LongInt_Array : LongErrArrayPtr    := RArray_Err;
      Float_Array   : FloatErrArrayPtr   := RArray_Err;
    END;

  IF (RArray_Err = NIL) THEN { RArray_Err is not there }
    FOR index := 0 TO N_Points-1 DO
    BEGIN
      { get the right kind of data, but I must turn off range checking for a moment }
{$R-}
      CASE MyType OF
        Byte_Array    : temp := ByteArrayPtr^[index];
        Word_Array    : temp := WordArrayPtr^[index];
        Integer_Array : temp := IntegerArrayPtr^[index];
        LongInt_Array : temp := LongArrayPtr^[index];
        Float_Array   : temp := FloatArrayPtr^[index];
      END;
{$R+}
      IF ((temp < 0) AND (Axis_Type = LOG)) THEN temp := -temp;
      IF (temp < x_min) THEN x_min := temp;
      IF (temp > x_max) THEN x_max := temp;
    END
  ELSE { hopefully, RArray_Err is there }
    FOR index := 0 TO N_Points-1 DO
    BEGIN
      { get the right kind of data, but I must turn off range checking for a moment }
{$R-}
      CASE MyType OF
        Byte_Array    : temp := ByteArrayPtr^[index] - ByteErrArrayPtr^[index];
        Word_Array    : temp := WordArrayPtr^[index] - WordErrArrayPtr^[index];
        Integer_Array : temp := IntegerArrayPtr^[index] - IntegerErrArrayPtr^[index];
        LongInt_Array : temp := LongArrayPtr^[index] - LongErrArrayPtr^[index];
        Float_Array   : temp := FloatArrayPtr^[index] - FloatErrArrayPtr^[index];
      END;
{$R+}
      IF ((temp < 0) AND (Axis_Type = LOG)) THEN temp := -temp;
      IF (temp < x_min) THEN x_min := temp;
{$R-}
      CASE MyType OF
        Byte_Array    : temp := ByteArrayPtr^[index] + ByteErrArrayPtr^[index];
        Word_Array    : temp := WordArrayPtr^[index] + WordErrArrayPtr^[index];
        Integer_Array : temp := IntegerArrayPtr^[index] + IntegerErrArrayPtr^[index];
        LongInt_Array : temp := LongArrayPtr^[index] + LongErrArrayPtr^[index];
        Float_Array   : temp := FloatArrayPtr^[index] + FloatErrArrayPtr^[index];
      END;
{$R+}
      IF (temp > x_max) THEN x_max := temp;
    END;
END; { of Limits_of_Array }


PROCEDURE Scale_Axis(RArray, RArray_Err :POINTER; MyType :DataType;
                     N_Points :INTEGER;
                     VAR AxisMin, AxisMax, MajorTic :FLOAT;
                     Axis_Type :LINLOG);
{
  A procedure which, given a data range, will compute "nice" tic marks
  for an axis of a graph to display the data range. This version requires
  a numeric coprocessor.
  Author: Marc Laliberte
  Published in TUG Lines - Turbo User Group, PO Box 1510, Poulsbo, WA 98370

  Modified by dbh to make it more robust and "un-crashable" (10/3/89)

  Inputs
    RArray - pointer to 1D data array
    RArray_Err - pointer to 1D array of error bar values for RArray
    MyType - element of enumerated field DataType that tells me what kind of
            a data array this is
    N_Points - number of data points
    Axis_Type - whether LIN or LOG axis
  Outputs:
    AxisMin - "nice" minimum (left) axis limit
    AxisMax - "nice" maximum (right) axis limit
    MajorTic - major tic separation
}
VAR
  DataMax, DataMin :FLOAT;
  deltax :FLOAT; { the data range :DataMax - DataMin }
  m :FLOAT; { the reciprocal of TicSeparation. Chosen so that the number of
              tic marks (about m * deltax) is between 4 and 10 }
  ex :LONGINT;
  mant :FLOAT; { base 10 exponent (ex) and mantissa (mant) for m. Chosen
                 so that m = mant * 10^ex has a "nice" value. }
  ntics :BYTE; { number of major tics; not used in this implementation }
  reversed :BOOLEAN; { flag to indicate if "min" and "max" were reversed
                       on input }
BEGIN
  { get minimum and maximum of data arrays }
  Limits_of_Array(RArray,RArray_Err,MyType,
    N_Points,Axis_Type,DataMin,DataMax);

  { calculate the data range }
  deltax := DataMax - DataMin;

  { let's check for pathological inputs and take action }
  IF (deltax > 0) THEN { DataMax > DataMin }
    reversed := FALSE
  ELSE
    IF (deltax = 0) THEN { the data array was all the same }
    BEGIN
      AxisMin := DataMin; { make up some nominal value }
      IF (AxisMin = 0) THEN AxisMin := 1;
      AxisMax := 2 * AxisMin;
      MajorTic := 1;
      Ntics := 5;
      Exit; { get out of here and go back }
    END
    ELSE { DataMax < DataMin !!! }
    BEGIN
      reversed := TRUE;
      deltax := -deltax;
      mant := DataMax; { use mant as temporary storage }
      DataMax := DataMin;
      DataMin := mant;
    END;

  IF (Axis_Type = LOG) THEN
  BEGIN
    IF (DataMin < 0) THEN
      DataMin := ABS(DataMin)
    ELSE IF (DataMin = 0) THEN
      DataMin := 1;
    ex := TRUNC(LOG10(DataMin));
    IF ((ex < 0) OR ((ex=0) AND (DataMin < 1))) THEN
      DEC(ex); { DataMin was < 1 }
    AxisMin := POW(10,ex);
    IF (DataMax < 0) THEN
      DataMax := ABS(DataMax)
      ELSE IF (DataMax = 0) THEN
        DataMax := 10;
    ex := TRUNC(LOG10(DataMax));
    IF (ex >= 0) THEN { DataMax was >= 1 }
      INC(ex);
    AxisMax := POW(10,ex);
    IF (AxisMin = AxisMax) THEN
      AxisMax := 2*AxisMin;
    MajorTic := 10; { this is ignored for LOG plots anyway }
    Exit;
  END; { IF (Axis_Type = LOG) }

  { set initial values for mant and ex for a linear axis }
  mant := 1.0;
  ex := 0;

  { the following lengthy IF computes a "nice" value for m = mant * 10^ex
    (note tha ex is an integer and mant can only take on the values 1.0,
    2.0, and 5.0) so that the number of tic-intervals required to cover an
    interval of length deltax is reasonable. }
  IF ((4.0 <= deltax) AND (deltax < 10.0)) THEN
    m := 1.0
  ELSE
    IF (deltax < 4.0) THEN
    REPEAT
      IF (mant < 1.5) THEN
        mant := 2.0
      ELSE
        IF (mant < 3.5) THEN
          mant := 5.0
        ELSE
        BEGIN
          mant := 1.0;
          INC(ex);
        END;
      m := mant * POW(10,ex);
    UNTIL (m * deltax >= 4.0)
    ELSE
    REPEAT
      IF (mant > 3.5) THEN
        mant := 2.0
      ELSE
        IF (mant > 1.5) THEN
          mant := 1.0
        ELSE
        BEGIN
          mant := 5.0;
          DEC(ex);
        END;
      m := mant * POW(10,ex);
    UNTIL (m * deltax < 10);

  { Compute the number of tics needed. Actually at this stage, NTics does
    not represent the number of tic marks needed, but the number of tic
    intervals. }
  NTics := TRUNC(m * deltax + 1.0);

  { find the (nice) left hand end of the axis }
  IF (DataMin < 0.0) THEN
    AxisMin := INT(DataMin * m - 0.99999999)/m
  ELSE
    AxisMin := INT(DataMin * m)/m;

  { Its possible that out computation of the number of intervals needed is
    off by one. We check that here and make a correction if necessary. }
  IF (AxisMin + NTics/m) < DataMax THEN
    INC(NTics);

  { finally, we compute the right hand end of the axis, increment NTics so
    that it counts the number of tics rather than the number of intervals,
    and figure out the distance between adjacent tics. }
  AxisMax := AxisMin + NTics/m;
  INC(NTics);
  MajorTic := 1/m;

  { now, finally remember that we might have reversed the order on input:
    ie DataMax < DataMin, so let's adjust results accordingly }
  IF (reversed) THEN
  BEGIN
    mant := AxisMax; { use mant again for temporary storage }
    AxisMax := AxisMin;
    AxisMin := mant;
  END;
END; { of Scale_Axis }

END.